unit uResultViewer1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, uPlot_ResultWriter, Controls, ComCtrls, StdCtrls, uMAADataTypes,
  uPlotClass, uPlotAxis, uPlotForms, uPlotRect, uPlotSeries, uPlotUtils, uPlotStyles,
  uPlotTemplates, Graphics, uPlotDataTypes;


type

  { TResultViewer }

  TResultViewer = class(TControl)
  private
    FHostControl: TWinControl;
    FResultReader: TPlot_ResultReaderMAAResult;
    function GetHostControl: TWinControl;
    function GetResultReader: TPlot_ResultReaderMAAResult;
    procedure SetHostControl(AValue: TWinControl);
    procedure SetResultReader(AValue: TPlot_ResultReaderMAAResult);
  protected
  public
    constructor Create; virtual;
    destructor Destroy; override;

    procedure Update; virtual;
    procedure Update(AResultIndex: Integer);virtual;
    property ResultReader: TPlot_ResultReaderMAAResult read GetResultReader write SetResultReader;
    property HostControl: TWinControl read GetHostControl write SetHostControl;
  end;

  { TResultViewer_Header }

  TResultViewer_Header = class(TResultViewer)
  private
    Flv_header: TListView;
  protected
  public
    constructor Create; override;
    destructor Destroy; override;

    procedure Update;override;
  end;

  { TResultViewer_List }

  TResultViewer_List = class(TResultViewer)
  private
    Flv_list: TListView;
  protected
  public
    constructor Create; override;
    destructor Destroy; override;

    procedure Update;override;
    procedure Update(AResultIndex: Integer);override;
  end;

  { TResultViewer_Plot }

  TResultViewer_Plot = class(TResultViewer)
  private
    FPlot: TPlot;
  protected
    procedure MakePlot(ANumSeries: Integer);
  public
    constructor Create; override;
    destructor Destroy; override;

    procedure Update;override;
    procedure Update(AResultIndex: Integer);override;
  end;


const
  cHeader_ItemNames : array [0..3] of string = ('Num Datasets','Date', 'Description','Resulttype');
  cCOLORS : array [0..3] of TColor = (clBlack, clRed, clYellow, clGreen);

implementation

{ TResultViewer_Plot } // =======================================================

procedure TResultViewer_Plot.MakePlot(ANumSeries: Integer);
 var
  vBorderAbs, vBorderRel : TBorder;
  vIndex : integer;
  vAxisList : TList;
  vPoint: TPoint;
  vViewRange: TValueRange;
  vCloneParams: TCloneAxisParams;
  vLoop: Integer;
begin
  FPLot.ClearAll;
  //UpdatePlotRects(Self);
  //UpdateSeries;
  // establish 2 PlotRects
  //TmplPlotRect(plot border legend colorscale);
  // 1st PlotRect --------------------------------------------------------
   vBorderRel.Left := 0;
   vBorderRel.Right := 0;
   vBorderRel.Top := 0;
   vBorderRel.Bottom := 00;
   vBorderAbs.Left := 04;
   vBorderAbs.Right := 04;
   vBorderAbs.Top := 04;
   vBorderAbs.Bottom := 04;
     vIndex := TmplPlotRect(FPlot, vBorderAbs, vBorderRel, FALSE, FALSE);
     TPlotRect(FPlot.PlotRect[vIndex]).Title := 'Spectrum (FFT)';
     // 1st set of axes
   try
     vAxisList := TmplAxes(FPlot, TPlotRect(FPlot.PlotRect[vIndex]), 2); // 2 axes
     TPlotAxis(FPlot.Axis[0]).AxisLabel := 'Frequency';
     TPlotAxis(FPlot.Axis[0]).NumberFormat := nfEngineeringPrefix;

     TPlotAxis(FPlot.Axis[1]).AxisLabel := 'Amplitude';
     TPlotAxis(FPlot.Axis[1]).NumberFormat := nfEngineering;

     TPlotAxis(FPlot.Axis[0]).AddInnerGridAxis(1);  // axis 0 draws inner ticks relative to axis 1
     TPlotAxis(FPlot.Axis[1]).AddInnerGridAxis(0);  // and vice versa

     TPlotAxis(FPlot.Axis[0]).InnerSubTicks:=FALSE;  // dont clutter the axis with too much subtcks
     TPlotAxis(FPlot.Axis[1]).InnerSubTicks:=TRUE; // and vice versa

     // 1st set of series FFTA
     for vLoop := 0 to ANumSeries-1 do begin
       vIndex := TmplSeries(FPlot, vAxisList, stSPECTRUM);  // 2 dimensions, waterfall=FALSE
       //writeln('created FfftA series idx ', IntToStr(vIndex));
       //FpsA := TXYSpectrumPlotSeries(FPlot.Series[vIndex]); // assign upper plotrect to fftA
       TXYSpectrumPlotSeries(FPlot.Series[vIndex]).AddUnit('x', '[Hz]');
       TXYSpectrumPlotSeries(FPlot.Series[vIndex]).AddUnit('y', '[dB]');
       TXYSpectrumPlotSeries(FPlot.Series[vIndex]).Caption := 'Spectrum';
       TXYSpectrumPlotSeries(FPlot.Series[vIndex]).AutoScaleMode := asFitNextMargined;

       //FPlot.Series[vIndex].Style := TSeriesStylePoints.Create;
       //TSeriesStylePoints(FPlot.Series[vIndex].Style).Diameter := 3;
       TXYSpectrumPlotSeries(FPlot.Series[vIndex]).Style := TSeriesStyleLines.Create;
       TSeriesStyleLines(TXYSpectrumPlotSeries(FPlot.Series[vIndex]).Style).Diameter := 5;
       TSeriesStyleLines(TXYSpectrumPlotSeries(FPlot.Series[vIndex]).Style).Color := cCOLORS[vLoop];
       TSeriesStyleLines(TXYSpectrumPlotSeries(FPlot.Series[vIndex]).Style).LineWidth := 3;
     end;

   finally
     // so gehts aufräumen der Axis List
     IF vAxisList <> nil THEN  begin
       while vAxisList.Count > 0 do begin // vAxisList.Remove(vAxisList.Items[0]);
         Freemem(vAxisList.Items[0]);
         vAxisList.Delete(0);
       end;
       vAxisList.Free;
     end;
   end;

   //// 2nd PlotRect ----------------------------------------------------------
   //none

  FPlot.TimedRefresh := TRUE;
  FPlot.Repaint;
end;

constructor TResultViewer_Plot.Create;
begin
  inherited Create;
  FPLot := TPlot.Create(Self);
  //FPLot.Parent := Self;
  //FPLot.Align := alClient;
  FPLot.Color := clBtnFace;
end;

destructor TResultViewer_Plot.Destroy;
begin
  inherited Destroy;
end;

procedure TResultViewer_Plot.Update;
begin
  inherited Update;
  Update(0);
end;

procedure TResultViewer_Plot.Update(AResultIndex: Integer);
var
  vError: Integer;
  vMeasResult: TMeasResult;
  vColumns, vRows: Integer;
  vDimCount, vPointCount: Integer;
  vPlotLine: TXYLine;
  vLoopSeries, vLoopDim, vLoop: Integer;
begin
  inherited Update(AResultIndex);
  Fplot.Parent := HostControl;
  FPLot.Align := alClient;
  // ClearPlot;
  FPlot.ClearAll;

  New(vMeasResult.lpMeasData);
  try
    vError := ResultReader.GetResultRecord(AResultIndex, vMeasResult);
    vDimCount := vMeasResult.dwResultIdent.diDimensions;
    vPointCount := vMeasResult.dwResultIdent.diPoints;
    // we need vColCount-1 series
    MakePlot(vDimCount-1);

    for vLoopSeries := 0 to vDimCount-2 do begin
      setlength(vPlotLine, vPointCount);
      try
        for vLoopDim := 1 to vDimCount-1 do begin
          for vLoop := 0 to vPointCount-1 do begin   // last point not relevant for fft ! might be 0/0
            vPlotLine[vLoop].X := vMeasResult.lpMeasData^[vLoop, 0];
            vPlotLine[vLoop].Y := vMeasResult.lpMeasData^[vLoop, vLoopDim];
          end;
        end;
        TXYSpectrumPlotSeries(FPlot.Series[vLoopSeries]).AddLine(vPlotLine);
      finally
        setlength(vPlotLine, 0);
      end;
    end;
    FPlot.AutoScalePlotRect(0);

  finally
    IF vMeasResult.lpMeasData <> nil THEN begin
      SetLength(vMeasResult.lpMeasData^, 0, 0);
      Dispose(vMeasResult.lpMeasData);
    end;
  end;
end;

{ TResultViewer_List } // =======================================================

constructor TResultViewer_List.Create;
begin
  inherited Create;
  Flv_list := TListView.Create(Self);
  with Flv_list do begin
    ViewStyle := vsReport;
    //Columns.Add; // item description
    //Columns.Add; // item value
    //Column[0].Caption := 'Dimension 0';
    //Column[1].Caption := 'Dimension 1';

    //Column[0].AutoSize:=TRUE;
    //Column[1].AutoSize:=TRUE;

    GridLines := TRUE;
    AutoWidthLastColumn := TRUE;
    AutoSize:=TRUE;
    ScrollBars := ssAutoBoth;
    ScrollBars:=ssAutoBoth;
    Parent := FHostControl;

    BorderSpacing.Right:=4;
    Align:=alClient;

    ReadOnly := true;
    RowSelect := true;
    //
    SortType := stNone;
    Visible := TRUE;
  end;
end;

destructor TResultViewer_List.Destroy;
begin
  inherited Destroy;
end;

procedure TResultViewer_List.Update;
begin
  inherited Update;
  Update(0);
end;

procedure TResultViewer_List.Update(AResultIndex: Integer);
var
  vError: Integer;
  vMeasResult: TMeasResult;
  vColumns, vRows: Integer;
  vColCount, vRowCount: Integer;
  vListItem: TListItem;
begin
  inherited Update(AResultIndex);
  Flv_list.Parent := HostControl;

  New(vMeasResult.lpMeasData);
  try
    Flv_list.Clear;
    vError := ResultReader.GetResultRecord(AResultIndex, vMeasResult);
    IF vError < 0 then exit;
    //------------------------
    vColCount := vMeasResult.dwResultIdent.diDimensions;
    vRowCount := vMeasResult.dwResultIdent.diPoints;

    //Flv_list.Clear;
    Flv_list.BeginUpdate;
      for vColumns:=0 to vColCount-1 do begin
        Flv_list.Columns.Add;
        Flv_list.Column[vColumns].Caption := 'Dim. '+IntToStr(vColumns);
      end;
      for vRows:=0 to vRowCount-1 do begin
        vListItem := Flv_list.Items.Add;
        vListItem.Caption := FloatToStrF(vMeasResult.lpMeasData^[vRows,0], ffExponent, 4, 4);
        for vColumns := 1 to vColCount-1 do begin
          vListItem.SubItems.Add( FloatToStrF(vMeasResult.lpMeasData^[vRows,vColumns], ffExponent, 4, 4) );
        end;
      end;
    Flv_list.EndUpdate;

  finally
    IF vMeasResult.lpMeasData <> nil THEN begin
      SetLength(vMeasResult.lpMeasData^, 0, 0);
      Dispose(vMeasResult.lpMeasData);
    end;
  end;


end;

{ TResultViewer_Header } // =======================================================

constructor TResultViewer_Header.Create;
begin
  Inherited Create;
  Flv_header := TListView.Create(Self);
  with Flv_header do begin
    ViewStyle := vsReport;
    Columns.Add; // item description
    Columns.Add; // item value
    Column[0].Caption := 'Header item';
    Column[1].Caption := 'value';

    Column[0].AutoSize:=TRUE;
    Column[1].AutoSize:=TRUE;

    GridLines := TRUE;
    AutoWidthLastColumn := TRUE;
    ScrollBars := ssAutoBoth;
    ScrollBars:=ssAutoBoth;
    Parent := FHostControl;

    BorderSpacing.Right:=4;
    Align:=alClient;

    ReadOnly := true;
    RowSelect := true;
    //
    SortType := stNone;
    Visible := TRUE;
  end;
end;

destructor TResultViewer_Header.Destroy;
begin
  inherited Destroy;
end;

procedure TResultViewer_Header.Update;
var
  vLoop: Integer;
  vLIstItem: TListItem;
  vResultTypeName: string;
begin
  inherited Update;
  Flv_header.Parent := FHostControl;

  Flv_header.Items.Clear;
  for vLoop := 0 to length(cHeader_ItemNames)-1 do begin
    vListItem := Flv_header.Items.Add;
    vListItem.Caption := (cHeader_ItemNames[vLoop]);
    vListItem.SubItems.Add('--');
  end;
  Flv_header.Items[0].SubItems.Strings[0] := IntToStr(ResultReader.NumResults);
  Flv_header.Items[1].SubItems.Strings[0] := DateTimeToStr(ResultReader.DataDateTime);
  Flv_header.Items[2].SubItems.Strings[0] := (ResultReader.DataDescription);
  if ResultReader.ResultType = ertMAAResult then vResultTypeName := 'MAAResult' else vResultTypeName := 'unknown';
  Flv_header.Items[3].SubItems.Strings[0] := vResultTypeName;
end;


{ TResultViewer } // =======================================================

function TResultViewer.GetHostControl: TWinControl;
begin
  Result := FHostControl;
end;

function TResultViewer.GetResultReader: TPlot_ResultReaderMAAResult;
begin
  Result := FResultReader;
end;

procedure TResultViewer.SetHostControl(AValue: TWinControl);
begin
  FHostControl := AValue;
end;

procedure TResultViewer.SetResultReader(AValue: TPlot_ResultReaderMAAResult);
begin
  FResultReader := AValue;
end;

constructor TResultViewer.Create;
begin

end;

destructor TResultViewer.Destroy;
begin
  inherited Destroy;
end;

procedure TResultViewer.Update;
begin
  IF FResultReader = nil then exit;
end;

procedure TResultViewer.Update(AResultIndex: Integer);
begin
  Inherited Update;
end;

end.

