unit uResultViewermain;

{
*****************************************************************************
*  This file is part of Multiple Acronym Math and Audio Plot - MAAPlot      *
*  MAA resultviewer                                                         *
*  for testing purposes only (might not be production stable)               *
*  uses: MAAplot V1.0                                                       *
*  License: EUPL V1.1                                                       *
*  (C) 2014 Stefan Junghans - www.junghans-electronics.de                   *
*****************************************************************************
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ComCtrls, ExtCtrls, Spin, uPlot_ResultWriter, uResultViewer1;

type

  { Tfrm_main }

  Tfrm_main = class(TForm)
    btn_fileopen: TButton;
    btn_export: TButton;
    ed_filename: TEdit;
    gb_file: TGroupBox;
    gb_header: TGroupBox;
    dlg_file: TOpenDialog;
    lbl_DataSet: TLabel;
    pc_data: TPageControl;
    ed_DataSet: TSpinEdit;
    ts_dataplot: TTabSheet;
    ts_datalist: TTabSheet;
    procedure btn_fileopenClick(Sender: TObject);
    procedure ed_DataSetChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure UpdateControls;
    procedure UpdateHeader;
    procedure UpdateList;
    procedure UpdatePlot;
  private
    { private declarations }
    FViewerHeader: TResultViewer_Header;
    FViewerList: TResultViewer_List;
    FViewerPlot: TResultViewer_Plot;
    FResultReader: TPlot_ResultReaderMAAResult;
  protected
  public
    { public declarations }
  end;

var
  frm_main: Tfrm_main;

implementation

{$R *.lfm}

{ Tfrm_main }

procedure Tfrm_main.btn_fileopenClick(Sender: TObject);
var
  vFileName: TFilename;
  vNumResults: Integer;
begin
  if dlg_file.Execute then vFileName:=dlg_file.FileName;
  try
    vNumResults := FResultReader.LoadFromFile(vFileName);      // TODO: deliver numresults ! (bug)
  except
    vNumResults:=-1;
  end;

  IF vNumResults > 0 then begin
    ed_filename.Text := vFileName;
    UpdateControls;
    UpdateHeader;
    UpdateList;
    UpdatePlot;
  end ELSE ed_filename.Text := 'error loading results';

end;

procedure Tfrm_main.ed_DataSetChange(Sender: TObject);
begin
  UpdateList;
  UpdatePlot;
end;

procedure Tfrm_main.FormCreate(Sender: TObject);
begin
   Self.Caption := 'MAA3 ResultViewer';
   dlg_file.Filter := 'MAAResult | *.mdt';
   FResultReader := TPlot_ResultReaderMAAResult.Create;
   //
   FViewerHeader := TResultViewer_Header.Create;
   FViewerHeader.HostControl := gb_header;
   FViewerHeader.ResultReader := FResultReader;
   //
   FViewerList := TResultViewer_List.Create;
   FViewerList.HostControl := ts_datalist;
   FViewerList.ResultReader := FResultReader;
   //
   FViewerPlot := TResultViewer_Plot.Create;
   FViewerPlot.HostControl := ts_dataplot;
   FViewerPlot.ResultReader := FResultReader;

   ed_DataSet.Enabled:=false;
end;

procedure Tfrm_main.FormDestroy(Sender: TObject);
begin
  IF FResultReader <> nil then FResultReader.Free;
  IF FViewerHeader <> nil then FViewerHeader.Free;
  IF FViewerList <> nil then FViewerList.Free;
end;

procedure Tfrm_main.UpdateControls;
begin
  ed_DataSet.MaxValue := FResultReader.NumResults;
  ed_DataSet.Enabled:=TRUE;
end;

procedure Tfrm_main.UpdateHeader;
begin
  IF FViewerHeader = nil then exit;
  FViewerHeader.Update;
end;

procedure Tfrm_main.UpdateList;
begin
  IF FViewerList = nil then exit;
  FViewerList.Update(ed_DataSet.Value-1);
end;

procedure Tfrm_main.UpdatePlot;
begin
  IF FViewerPlot = nil then exit;
  FViewerPlot.Update(ed_DataSet.Value-1);
end;

end.

